"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DepositAction = void 0;
const web3_js_1 = require("@solana/web3.js");
const spl_token_1 = require("@solana/spl-token");
const account_1 = require("../../utils/account");
const refreshReserve_1 = require("./util/refreshReserve");
const depositObligationCollateral_1 = require("./util/depositObligationCollateral");
const depositReserveLiquidity_1 = require("./util/depositReserveLiquidity");
const constant_1 = require("../../constant");
const initMining_1 = require("./util/initMining");
const depositMining_1 = require("./util/depositMining");
const larixInstructionCollector_1 = require("../../utils/larixInstructionCollector");
const baseAction_1 = require("./util/baseAction");
class DepositAction extends baseAction_1.BaseAction {
    deposit(amount, depositReserve, larixMining, larixObligation) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const lendingProgramID = this.marketConfig.programId;
            let mining = larixMining === null || larixMining === void 0 ? void 0 : larixMining.state;
            let obligation = larixObligation === null || larixObligation === void 0 ? void 0 : larixObligation.state;
            const liquidityTokenAccount = this.userInfo.getLiquidityTokenAccount(depositReserve);
            if (liquidityTokenAccount == undefined) {
                throw new Error("Invalid liquidity token account");
            }
            let collateralTokenAccountPubkey = (_a = this.userInfo.getCollateralTokenAccount(depositReserve)) === null || _a === void 0 ? void 0 : _a.pubkey;
            const larixTransaction = new larixInstructionCollector_1.LarixInstructionCollector(this.wallet);
            if (mining === undefined) {
                const myMinings = yield this.userInfo.getMiningOfMarket(this.marketConfig.marketId, this.marketConfig.programId);
                if (myMinings.length > 0) {
                    mining = myMinings[0];
                }
            }
            if (obligation == undefined) {
                const myObligations = yield this.userInfo.getObligationOfMarket(this.marketConfig.marketId, this.marketConfig.programId);
                if (myObligations.length > 0) {
                    obligation = myObligations[0];
                }
            }
            const balanceNeeded = yield spl_token_1.Token.getMinBalanceRentForExemptAccount(this.connection);
            const sourcePubkey = (0, account_1.ensureSplAccount)(larixTransaction.instructions, larixTransaction.cleanupInstructions, liquidityTokenAccount, this.userInfo.publicKey, amount.toNumber() + balanceNeeded, larixTransaction.signers);
            if (depositReserve.state.pubkey.toString() == "GaX5diaQz7imMTeNYs5LPAHX6Hq1vKtxjBYzLkjXipMh") {
                yield (0, account_1.getOrCreateAssociatedTokenAccount)(this.connection, larixTransaction.instructions, this.userInfo.publicKey, new web3_js_1.PublicKey("MNDEFzGvMt87ueuHvVU9VcTqsAP5b3fTGPsHuuPA5ey"), this.userInfo.publicKey);
            }
            else if (depositReserve.state.pubkey.toString() == "FStv7oj29DghUcCRDRJN9sEkB4uuh4SqWBY9pvSQ4Rch") {
                yield (0, account_1.getOrCreateAssociatedTokenAccount)(this.connection, larixTransaction.instructions, this.userInfo.publicKey, new web3_js_1.PublicKey("HZRCwxP2Vq9PCpPXooayhJ2bxTpo5xfpQrwB1svh332p"), this.userInfo.publicKey);
            }
            let miningID;
            yield (0, refreshReserve_1.refreshReserve)(this.connection, this.marketConfig, larixTransaction.signers, larixTransaction.instructions, larixTransaction.cleanupInstructions, depositReserve.state, lendingProgramID);
            collateralTokenAccountPubkey = yield (0, depositReserveLiquidity_1.depositReserveLiquidity)(this.connection, larixTransaction.signers, larixTransaction.instructions, larixTransaction.cleanupInstructions, sourcePubkey, amount, this.userInfo.publicKey, depositReserve.state, lendingProgramID, this.marketConfig.authorityId, collateralTokenAccountPubkey);
            if (mining == undefined) {
                miningID = yield (0, initMining_1.initMining)(this.connection, larixTransaction.signers, larixTransaction.instructions, larixTransaction.cleanupInstructions, this.marketConfig.marketId, this.wallet.publicKey, lendingProgramID);
            }
            else {
                miningID = mining.pubkey;
            }
            if (obligation != undefined) {
                let opened = false;
                obligation.info.deposits.map((collateralDetail) => {
                    if (collateralDetail.depositReserve.equals(depositReserve.state.pubkey)) {
                        opened = true;
                    }
                });
                if (opened) {
                    yield (0, refreshReserve_1.refreshReserve)(this.connection, this.marketConfig, larixTransaction.signers, larixTransaction.instructions, larixTransaction.cleanupInstructions, depositReserve.state, lendingProgramID);
                    yield (0, depositObligationCollateral_1.depositObligationCollateral)(this.connection, larixTransaction.signers, larixTransaction.instructions, larixTransaction.cleanupInstructions, this.userInfo.publicKey, constant_1.U64_MAX, collateralTokenAccountPubkey, depositReserve.state, obligation, lendingProgramID, this.marketConfig.authorityId);
                }
                else {
                    yield (0, refreshReserve_1.refreshReserve)(this.connection, this.marketConfig, larixTransaction.signers, larixTransaction.instructions, larixTransaction.cleanupInstructions, depositReserve.state, lendingProgramID);
                    yield (0, depositMining_1.depositMining)(this.connection, larixTransaction.signers, larixTransaction.instructions, larixTransaction.cleanupInstructions, collateralTokenAccountPubkey, depositReserve.state, constant_1.U64_MAX, this.userInfo.publicKey, this.marketConfig.marketId, miningID, lendingProgramID, this.marketConfig.authorityId);
                }
            }
            else {
                yield (0, refreshReserve_1.refreshReserve)(this.connection, this.marketConfig, larixTransaction.signers, larixTransaction.instructions, larixTransaction.cleanupInstructions, depositReserve.state, lendingProgramID);
                yield (0, depositMining_1.depositMining)(this.connection, larixTransaction.signers, larixTransaction.instructions, larixTransaction.cleanupInstructions, collateralTokenAccountPubkey, depositReserve.state, constant_1.U64_MAX, this.userInfo.publicKey, this.marketConfig.marketId, miningID, lendingProgramID, this.marketConfig.authorityId);
            }
            return larixTransaction;
        });
    }
}
exports.DepositAction = DepositAction;
