"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createClaimMiningMineInstruction = void 0;
const web3_js_1 = require("@solana/web3.js");
const buffer_layout_1 = __importDefault(require("buffer-layout"));
const instruction_1 = require("./instruction");
const config_1 = require("../../config");
const createClaimMiningMineInstruction = (mining, miningSupply, destination, miningOwner, lendingMarket, lendingMarketAuthority, marketProgramId) => {
    const dataLayout = buffer_layout_1.default.struct([
        buffer_layout_1.default.u8('instruction'),
    ]);
    const data = Buffer.alloc(dataLayout.span);
    dataLayout.encode({
        instruction: instruction_1.LendingInstruction.ClaimMiningMine,
    }, data);
    const keys = [
        { pubkey: mining.pubkey, isSigner: false, isWritable: true },
        { pubkey: miningSupply, isSigner: false, isWritable: true },
        { pubkey: destination, isSigner: false, isWritable: true },
        { pubkey: miningOwner, isSigner: true, isWritable: false },
        { pubkey: lendingMarket, isSigner: false, isWritable: false },
        { pubkey: lendingMarketAuthority, isSigner: false, isWritable: false },
        { pubkey: config_1.TOKEN_PROGRAM_ID, isSigner: false, isWritable: false },
    ];
    mining.info.miningIndices.map((item) => {
        keys.push({ pubkey: item.reserve, isSigner: false, isWritable: false });
    });
    return new web3_js_1.TransactionInstruction({
        keys,
        programId: marketProgramId,
        data,
    });
};
exports.createClaimMiningMineInstruction = createClaimMiningMineInstruction;
