/// <reference types="node" />
import { AccountInfo, PublicKey } from '@solana/web3.js';
import BN from 'bn.js';
import * as BufferLayout from 'buffer-layout';
import { LastUpdate } from './lastUpdate';
import { State } from "../index";
export interface Obligation {
    version: number;
    lastUpdate: LastUpdate;
    lendingMarket: PublicKey;
    owner: PublicKey;
    deposits: ObligationCollateral[];
    borrows: ObligationLiquidity[];
    depositedValue: BN;
    borrowedValue: BN;
    allowedBorrowValue: BN;
    unhealthyBorrowValue: BN;
    unclaimedMine: BN;
}
export interface ObligationCollateral {
    index: BN;
    depositReserve: PublicKey;
    depositedAmount: BN;
    marketValue: BN;
}
export interface ObligationLiquidity {
    index: BN;
    borrowReserve: PublicKey;
    cumulativeBorrowRateWads: BN;
    borrowedAmountWads: BN;
    marketValue: BN;
}
export declare const ObligationLayout: typeof BufferLayout.Structure;
export declare const ObligationCollateralLayout: typeof BufferLayout.Structure;
export declare const ObligationLiquidityLayout: typeof BufferLayout.Structure;
export declare const isObligation: (info: AccountInfo<Buffer>) => boolean;
export declare const ObligationParser: (pubkey: PublicKey, info: AccountInfo<Buffer>) => State<Obligation>;
