"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObligationParser = exports.isObligation = exports.ObligationLiquidityLayout = exports.ObligationCollateralLayout = exports.ObligationLayout = void 0;
const BufferLayout = __importStar(require("buffer-layout"));
const Layout = __importStar(require("../../utils/layout"));
const lastUpdate_1 = require("./lastUpdate");
exports.ObligationLayout = BufferLayout.struct([
    BufferLayout.u8('version'),
    lastUpdate_1.LastUpdateLayout,
    Layout.publicKey('lendingMarket'),
    Layout.publicKey('owner'),
    Layout.uint128('depositedValue'),
    Layout.uint128('borrowedValue'),
    Layout.uint128('allowedBorrowValue'),
    Layout.uint128('unhealthyBorrowValue'),
    BufferLayout.u8('depositsLen'),
    BufferLayout.u8('borrowsLen'),
    Layout.uint128("unclaimedMine"),
    BufferLayout.blob(96 * 9 + 72, 'dataFlat'),
]);
exports.ObligationCollateralLayout = BufferLayout.struct([
    Layout.publicKey('depositReserve'),
    Layout.uint64('depositedAmount'),
    Layout.uint128('marketValue'),
    Layout.uint128("index"),
]);
exports.ObligationLiquidityLayout = BufferLayout.struct([
    Layout.publicKey('borrowReserve'),
    Layout.uint128('cumulativeBorrowRateWads'),
    Layout.uint128('borrowedAmountWads'),
    Layout.uint128('marketValue'),
    Layout.uint128("index"),
]);
const isObligation = (info) => {
    return info.data.length === exports.ObligationLayout.span;
};
exports.isObligation = isObligation;
const ObligationParser = (pubkey, info) => {
    const buffer = Buffer.from(info.data);
    const { version, lastUpdate, lendingMarket, owner, depositedValue, borrowedValue, allowedBorrowValue, unhealthyBorrowValue, depositsLen, borrowsLen, unclaimedMine, dataFlat, } = exports.ObligationLayout.decode(buffer);
    if (lastUpdate.slot.isZero()) {
        throw new Error("lastUpdate.slot.isZero()");
    }
    const depositsBuffer = dataFlat.slice(0, depositsLen * exports.ObligationCollateralLayout.span);
    const deposits = BufferLayout.seq(exports.ObligationCollateralLayout, depositsLen).decode(depositsBuffer);
    const borrowsBuffer = dataFlat.slice(depositsBuffer.length, depositsBuffer.length + borrowsLen * exports.ObligationLiquidityLayout.span);
    const borrows = BufferLayout.seq(exports.ObligationLiquidityLayout, borrowsLen).decode(borrowsBuffer);
    const obligation = {
        version,
        lastUpdate,
        lendingMarket,
        owner,
        depositedValue,
        borrowedValue,
        allowedBorrowValue,
        unhealthyBorrowValue,
        unclaimedMine,
        deposits,
        borrows,
    };
    const details = {
        pubkey,
        account: Object.assign({}, info),
        info: obligation,
    };
    return details;
};
exports.ObligationParser = ObligationParser;
