/// <reference types="node" />
import { AccountInfo, PublicKey } from '@solana/web3.js';
import BN from 'bn.js';
import * as BufferLayout from 'buffer-layout';
import { LastUpdate } from './lastUpdate';
import { State } from "./index";
import BigNumber from "bignumber.js";
export interface Reserve {
    version: number;
    lastUpdate: LastUpdate;
    lendingMarket: PublicKey;
    liquidity: Liquidity;
    collateral: Collateral;
    config: Config;
    bonus: Bonus;
    bridgePool: any;
    depositLimit: number;
    isLP: boolean;
}
export interface LpInfo {
    token0Symbol: string;
    token1Symbol: string;
    perBlock: BigNumber;
    perBlockB: BigNumber;
    rewardASymbol: string;
    rewardBSymbol: string;
    farmPoolLpSupply: BigNumber;
    lpApr: BigNumber;
    lpFeesApr: number;
    lpRewardApyA: BigNumber;
    lpRewardApyB: BigNumber;
    doubleRewardLogoSource: any;
    doubleRewardApy: BigNumber;
    doubleRewardSymbol: string;
    amm_id: PublicKey;
}
export interface Liquidity {
    mintPubkey: PublicKey;
    mintDecimals: number;
    supplyPubkey: PublicKey;
    feeReceiver: PublicKey;
    usePythOracle: boolean;
    params_1: PublicKey;
    params_2: PublicKey;
    availableAmount: BN;
    borrowedAmountWads: BN;
    cumulativeBorrowRateWads: BN;
    marketPrice: BN | BigNumber;
    ownerUnclaimed: BN;
}
export interface Collateral {
    mintPubkey: PublicKey;
    mintTotalSupply: BN;
    supplyPubkey: PublicKey;
}
export interface Config {
    optimalUtilizationRate: number;
    loanToValueRatio: number;
    liquidationBonus: number;
    liquidationThreshold: number;
    minBorrowRate: number;
    optimalBorrowRate: number;
    maxBorrowRate: number;
    fees: {
        borrowFeeWad: BN;
        borrowInterestFeeWad: BN;
        flashLoanFeeWad: BN;
        hostFeePercentage: number;
    };
    depositPaused: boolean;
    borrowPaused: boolean;
    liquidationPaused: boolean;
}
export interface Bonus {
    unCollSupply: PublicKey;
    lTokenMiningIndex: BN;
    borrowMiningIndex: BN;
    /**
     * 当前Reserve每个slot产出的Larix矿的数量，没0.4s一个Slot，该数的低6位为小数位
     */
    totalMiningSpeed: BN;
    kinkUtilRate: BN;
}
export declare const ReserveLayout: typeof BufferLayout.Structure;
export declare const isReserve: (info: AccountInfo<Buffer>) => boolean;
export declare const ReserveParser: (pubkey: PublicKey, info: AccountInfo<Buffer> | null) => State<Reserve>;
